#ifndef ENGINE_H
#define ENGINE_H

#include <sys/fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/soundcard.h>
#include <limits.h>

#define MONO 1
#define STEREO 2
#define PLAY 3
#define MUTE 4
#define REC 5
#define PREC 6
#define NOSPACE 7

#ifdef __cplusplus
extern "C" {
#endif
struct _channel{
  int chan;
  int left;
  int right;
  int type; /* MONO or STEREO? */
  int state; /* PLAY, MUTE, REC or PREC */
};

extern struct _channel devs[SOUND_MIXER_NRDEVICES + 1];

extern int mixer_fd;
extern const char ret[10][151];

extern int recmask;
extern int devmask;
extern int recscr;
extern int stereomask;

int *readFromDevice(const int device);
int writeToDevice(const int device, const int left, const int right);
void getChannels(void);
int getState(const int device, const int *vol);
int searchDevice(const char *nameOfDevice);
int checkSpace(const char *buf);
int writeFile(const char *name);
int parseFile(const char *filename);
int deleteFile(const char *name);
int isRecordable(const int device);
int isRecording(const int device);
int modifySrc(const int device, const int caso);
void initMixer(void);
int initMask(void);
int getProfiles(void);
void err_quit(const char *msg);
#ifdef __cplusplus
}
#endif

#endif
